@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -161,10 +161,6 @@ gpk_auto_refresh_get_frequency_prefs (GpkAutoRefresh *arefresh, const gchar *key
 
 	/* get from gconf */
 	value = gconf_client_get_int (arefresh->priv->gconf_client, key, NULL);
-	if (value == 0) {
-		egg_warning ("no schema for %s, using daily", key);
-		value = 86400;
-	}
 
 	/* convert to enum and get seconds */
 	return value;
