@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -69,18 +69,37 @@ gpk_helper_deps_install_show (GpkHelperDepsInstall *helper, PkPackageList *packa
 	GtkResponseType response;
 	gchar *package_id;
 	const PkPackageObj *obj;
-	guint i;
+	const PkPackageObj *obj_tmp;
+	guint i = 0, j;
 
 	/* remove cleanup packages */
-	length = pk_package_list_get_size (deps_list);
-	for (i=0; i<length; i++) {
+	while (i<pk_package_list_get_size (deps_list)) {
 		obj = pk_package_list_get_obj (deps_list, i);
 		if (obj->info == PK_INFO_ENUM_CLEANUP ||
 		    obj->info == PK_INFO_ENUM_FINISHED) {
 			package_id = pk_package_id_to_string (obj->id);
 			pk_package_list_remove (deps_list, package_id);
 			g_free (package_id);
+			continue;
 		}
+
+		/* remove original packages */
+		ret = FALSE;
+		length = pk_package_list_get_size (packages);
+		for (j=0; j<length; j++) {
+			obj_tmp = pk_package_list_get_obj (packages, j);
+			if (pk_package_id_equal (obj_tmp->id, obj->id)) {
+				package_id = pk_package_id_to_string (obj->id);
+				pk_package_list_remove (deps_list, package_id);
+				g_free (package_id);
+				ret = TRUE;
+			}
+		}
+		if (ret)
+			continue;
+
+		/* only increment if we didn't remove a package */
+		i++;
 	}
 
 	/* empty list */
