@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -37,11 +37,11 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <libnotify/notify.h>
-#include <polkit-gnome/polkit-gnome.h>
 #include <packagekit-glib/packagekit.h>
 
 #include "egg-debug.h"
 #include "egg-string.h"
+#include "egg-console-kit.h"
 
 #include "gpk-common.h"
 #include "gpk-session.h"
@@ -49,7 +49,6 @@
 #include "gpk-watch.h"
 #include "gpk-modal-dialog.h"
 #include "gpk-inhibit.h"
-#include "gpk-consolekit.h"
 #include "gpk-enum.h"
 
 static void     gpk_watch_finalize	(GObject       *object);
@@ -74,10 +73,10 @@ struct GpkWatchPrivate
 	PkTaskList		*tlist;
 	PkRestartEnum		 restart;
 	GConfClient		*gconf_client;
-	PolKitGnomeAction	*restart_action;
 	guint			 set_proxy_timeout;
 	gchar			*error_details;
 	gboolean		 hide_warning;
+	EggConsoleKit		*console;
 };
 
 typedef struct {
@@ -120,6 +119,62 @@ gpk_watch_cached_message_free (GpkWatchCachedMessage *cached_message)
 	g_free (cached_message);
 }
 
+/**
+ * gpk_watch_get_restart_required_tooltip:
+ **/
+static gchar *
+gpk_watch_get_restart_required_tooltip (GpkWatch *watch)
+{
+	gchar *package_loc = NULL;
+	gchar **packages = NULL;
+	guint len;
+	const gchar *title;
+	gchar *message = NULL;
+	gchar *text = NULL;
+
+	/* nothing */
+	if (watch->priv->restart == PK_RESTART_ENUM_NONE)
+		goto out;
+
+	/* size */
+	len = watch->priv->restart_package_names->len;
+	if (len == 0)
+		goto out;
+
+	/* localised title */
+	title = gpk_restart_enum_to_localised_text (watch->priv->restart);
+
+	/* non-security require */
+	if (watch->priv->restart == PK_RESTART_ENUM_SESSION ||
+	    watch->priv->restart == PK_RESTART_ENUM_SYSTEM) {
+
+		/* get localised list */
+		packages = pk_ptr_array_to_strv (watch->priv->restart_package_names);
+		package_loc = gpk_strv_join_locale (packages);
+		if (package_loc != NULL) {
+			/* TRANSLATORS: a list of packages is shown that need to restarted */
+			message = g_strdup_printf (ngettext ("This is due to the %s package being updated.",
+							     "This is due to the following packages being updated: %s.", len), package_loc);
+		} else {
+			/* TRANSLATORS: over 5 packages require the system to be restarted, don't list them all here */
+			message = g_strdup_printf (ngettext ("This is because %i package has been updated.",
+							     "This is because %i packages have been updated.", len), len);
+		}
+
+		/* join */
+		text = g_strdup_printf ("%s\n%s", title, message);
+		goto out;
+	}
+
+	/* just use title, as security requires are not the package that are updated */
+	text = g_strdup (title);
+out:
+	g_strfreev (packages);
+	g_free (package_loc);
+	g_free (message);
+	return text;
+}
+
 /**
  * gpk_watch_refresh_tooltip:
  **/
@@ -133,8 +188,7 @@ gpk_watch_refresh_tooltip (GpkWatch *watch)
 	GString *status;
 	const gchar *trailer;
 	const gchar *localised_status;
-	gchar *package_loc;
-	gchar **packages;
+	gchar *text;
 
 	g_return_val_if_fail (GPK_IS_WATCH (watch), FALSE);
 
@@ -144,40 +198,19 @@ gpk_watch_refresh_tooltip (GpkWatch *watch)
 	if (length == 0) {
 
 		/* any restart required? */
-		if (watch->priv->restart != PK_RESTART_ENUM_NONE) {
-			g_string_append (status, gpk_restart_enum_to_localised_text (watch->priv->restart));
-			g_string_append_c (status, '\n');
-
-			len = watch->priv->restart_package_names->len;
-
-			if (len > 0) {
-				packages = pk_ptr_array_to_strv (watch->priv->restart_package_names);
-				package_loc = gpk_strv_join_locale (packages);
-				g_strfreev (packages);
-				if (package_loc != NULL) {
-					/* TRANSLATORS: a list of packages is shown that need to restarted */
-					g_string_append_printf (status, ngettext ("This is due to the %s package being updated.",
-										  "This is due to the following packages being updated: %s", len), package_loc);
-				} else {
-					/* TRANSLATORS: over 5 packages require the system to be restarted, don't list them all here */
-					g_string_append_printf (status, ngettext ("This is because %i package has been updated.",
-										  "This is because %i packages have been updated.", len), len);
-				}
-				g_string_append_c (status, '\n');
-				g_free (package_loc);
-			}
-
-			/* remove final \n */
-			if (status->len > 0)
-				g_string_set_size (status, status->len - 1);
-
-			goto out;
+		text = gpk_watch_get_restart_required_tooltip (watch);
+		if (text != NULL) {
+			g_string_append (status, text);
 		}
+		g_free (text);
 
 		/* do we have any cached messages to show? */
 		len = watch->priv->cached_messages->len;
 		if (len > 0) {
-			g_string_append_printf (status, ngettext ("%i message from the package manager", "%i messages from the package manager", len), len);
+			if (status->len > 0)
+				g_string_append_c (status, '\n');
+			g_string_append_printf (status, ngettext ("%i message from the package manager",
+								  "%i messages from the package manager", len), len);
 			goto out;
 		}
 
@@ -334,7 +367,7 @@ gpk_watch_refresh_icon (GpkWatch *watch)
 	/* any restart required? */
 	if (watch->priv->restart != PK_RESTART_ENUM_NONE &&
 	    watch->priv->hide_warning == FALSE) {
-		icon_name = gpk_restart_enum_to_dialog_icon_name (watch->priv->restart);
+		icon_name = gpk_restart_enum_to_icon_name (watch->priv->restart);
 		goto out;
 	}
 
@@ -376,11 +409,11 @@ gpk_watch_libnotify_cb (NotifyNotification *notification, gchar *action, gpointe
 {
 	GpkWatch *watch = GPK_WATCH (data);
 
-	if (egg_strequal (action, "do-not-show-notify-complete")) {
+	if (g_strcmp0 (action, "do-not-show-notify-complete") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_COMPLETED);
 		gconf_client_set_bool (watch->priv->gconf_client, GPK_CONF_NOTIFY_COMPLETED, FALSE, NULL);
 
-	} else if (egg_strequal (action, "show-error-details")) {
+	} else if (g_strcmp0 (action, "show-error-details") == 0) {
 		/* TRANSLATORS: The detailed error if the user clicks "more info" */
 		gpk_error_dialog (_("Error details"), _("Package manager error details"), watch->priv->error_details);
 
@@ -405,7 +438,9 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 	gchar *message = NULL;
 	NotifyNotification *notification;
 #if PK_CHECK_VERSION(0,5,0)
+	guint j;
 	const PkRequireRestartObj *obj;
+	const PkRequireRestartObj *obj_tmp;
 	PkObjList *array;
 #else
 	PkPackageId *id;
@@ -422,6 +457,12 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 	}
 	egg_debug ("role=%s, text=%s", pk_role_enum_to_text (role), text);
 
+	/* is it worth processing? */
+	if (exit_enum != PK_EXIT_ENUM_SUCCESS) {
+		egg_debug ("not processing, as didn't complete okay");
+		goto out;
+	}
+
 	/* show an icon if the user needs to reboot */
 	if (role == PK_ROLE_ENUM_UPDATE_PACKAGES ||
 	    role == PK_ROLE_ENUM_INSTALL_PACKAGES ||
@@ -438,8 +479,25 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 			}
 			for (i=0; i<array->len; i++) {
 				obj = pk_obj_list_index (array, i);
-				if (obj->restart >= restart)
-					g_ptr_array_add (watch->priv->restart_package_names, g_strdup (obj->id->name));
+
+				/* is a lesser restart that what we have already */
+				if (obj->restart != restart)
+					continue;
+
+				/* is already in the list */
+				ret = FALSE;
+				for (j=0; j<array->len; j++) {
+					obj_tmp = pk_obj_list_index (array, j);
+					if (g_strcmp0 (obj_tmp->id->name, obj->id->name) == 0) {
+						ret = TRUE;
+						break;
+					}
+				}
+				if (ret)
+					continue;
+
+				/* add to list */
+				g_ptr_array_add (watch->priv->restart_package_names, g_strdup (obj->id->name));
 			}
 			g_object_unref (array);
 no_data:
@@ -462,12 +520,6 @@ no_data:
 		goto out;
 	}
 
-	/* is it worth showing a UI? */
-	if (exit_enum != PK_EXIT_ENUM_SUCCESS) {
-		egg_debug ("not notifying, as didn't complete okay");
-		goto out;
-	}
-
 	/* are we accepting notifications */
 	value = gconf_client_get_bool (watch->priv->gconf_client, GPK_CONF_NOTIFY_COMPLETED, NULL);
 	if (!value) {
@@ -732,17 +784,17 @@ gpk_watch_show_about_cb (GtkMenuItem *item, gpointer data)
 		NULL};
 	const char *license[] = {
 		N_("Licensed under the GNU General Public License Version 2"),
-		N_("PackageKit is free software; you can redistribute it and/or\n"
-		   "modify it under the terms of the GNU General Public License\n"
-		   "as published by the Free Software Foundation; either version 2\n"
+		N_("PackageKit is free software; you can redistribute it and/or "
+		   "modify it under the terms of the GNU General Public License "
+		   "as published by the Free Software Foundation; either version 2 "
 		   "of the License, or (at your option) any later version."),
-		N_("PackageKit is distributed in the hope that it will be useful,\n"
-		   "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+		N_("PackageKit is distributed in the hope that it will be useful, "
+		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
 		   "GNU General Public License for more details."),
-		N_("You should have received a copy of the GNU General Public License\n"
-		   "along with this program; if not, write to the Free Software\n"
-		   "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA\n"
+		N_("You should have received a copy of the GNU General Public License "
+		   "along with this program; if not, write to the Free Software "
+		   "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA "
 		   "02110-1301, USA.")
 	};
 	const char *translators = _("translator-credits");
@@ -763,6 +815,7 @@ gpk_watch_show_about_cb (GtkMenuItem *item, gpointer data)
 			       "version", VERSION,
 			       "copyright", "Copyright \xc2\xa9 2007-2009 Richard Hughes",
 			       "license", license_trans,
+			       "wrap-license", TRUE,	
 			       "website-label", _("PackageKit Website"),
 			       "website", "www.packagekit.org",
 			       "comments", "PackageKit",
@@ -806,15 +859,6 @@ gpk_watch_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 times
 		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 }
 
-/**
- * gpk_watch_restart_cb:
- **/
-static void
-gpk_watch_restart_cb (PolKitGnomeAction *action, gpointer data)
-{
-	gpk_restart_system ();
-}
-
 /**
  * gpk_watch_menu_show_messages_cb:
  **/
@@ -938,29 +982,22 @@ static gchar *
 gpk_watch_get_role_text (PkClient *client)
 {
 	const gchar *role_text;
-	gchar *text;
 	gchar *message;
 	PkRoleEnum role;
 	GError *error = NULL;
 	gboolean ret;
 
 	/* get role and text */
-	ret = pk_client_get_role (client, &role, &text, &error);
+	ret = pk_client_get_role (client, &role, NULL, &error);
 	if (!ret) {
 		egg_warning ("failed to get role: %s", error->message);
 		g_error_free (error);
 		return NULL;
 	}
 
-	/* backup */
+	/* present tense localisation */
 	role_text = gpk_role_enum_to_localised_present (role);
-
-	if (!egg_strzero (text) && role != PK_ROLE_ENUM_UPDATE_PACKAGES)
-		message = g_strdup_printf ("%s: %s", role_text, text);
-	else
-		message = g_strdup_printf ("%s", role_text);
-	g_free (text);
-
+	message = g_strdup_printf ("%s", role_text);
 	return message;
 }
 
@@ -1246,6 +1283,23 @@ gpk_watch_menu_log_out_cb (GtkMenuItem *item, gpointer data)
 	g_object_unref (session);
 }
 
+/**
+ * gpk_watch_menu_restart_cb:
+ **/
+static void
+gpk_watch_menu_restart_cb (GtkMenuItem *item, GpkWatch *watch)
+{
+	gboolean ret;
+	GError *error = NULL;
+
+	/* restart using ConsoleKit */
+	ret = egg_console_kit_restart (watch->priv->console, &error);
+	if (!ret) {
+		egg_warning ("restarting failed: %s", error->message);
+		g_error_free (error);
+	}
+}
+
 /**
  * gpk_watch_activate_status_cb:
  * @button: Which buttons are pressed
@@ -1260,6 +1314,7 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 	GtkWidget *image;
 	guint len;
 	gboolean show_hide = FALSE;
+	gboolean can_restart = FALSE;
 
 	g_return_if_fail (GPK_IS_WATCH (watch));
 
@@ -1282,7 +1337,9 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 	}
 
 	/* log out session */
-	if (watch->priv->restart == PK_RESTART_ENUM_SESSION) {
+	if (watch->priv->restart == PK_RESTART_ENUM_SESSION ||
+	    watch->priv->restart == PK_RESTART_ENUM_SECURITY_SESSION) {
+		/* TRANSLATORS: log out of the session */
 		widget = gtk_image_menu_item_new_with_mnemonic (_("_Log out"));
 		image = gtk_image_new_from_icon_name ("system-log-out", GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
@@ -1293,8 +1350,16 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 	}
 
 	/* restart computer */
-	if (watch->priv->restart == PK_RESTART_ENUM_SYSTEM) {
-		widget = gtk_action_create_menu_item (GTK_ACTION (watch->priv->restart_action));
+	egg_console_kit_can_restart (watch->priv->console, &can_restart, NULL);
+	if (can_restart &&
+	    (watch->priv->restart == PK_RESTART_ENUM_SYSTEM ||
+	     watch->priv->restart == PK_RESTART_ENUM_SECURITY_SYSTEM)) {
+		/* TRANSLATORS: this menu item restarts the computer after an update */
+		widget = gtk_image_menu_item_new_with_mnemonic (_("_Restart computer"));
+		image = gtk_image_new_from_icon_name ("system-shutdown", GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
+		g_signal_connect (G_OBJECT (widget), "activate",
+				  G_CALLBACK (gpk_watch_menu_restart_cb), watch);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), widget);
 		show_hide = TRUE;
 	}
@@ -1317,22 +1382,6 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 			1, gtk_get_current_event_time());
 }
 
-/**
- * pk_connection_changed_cb:
- **/
-static void
-pk_connection_changed_cb (PkConnection *pconnection, gboolean connected, GpkWatch *watch)
-{
-	g_return_if_fail (GPK_IS_WATCH (watch));
-	egg_debug ("connected=%i", connected);
-	if (connected) {
-		gpk_watch_refresh_icon (watch);
-		gpk_watch_refresh_tooltip (watch);
-	} else {
-		gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
-	}
-}
-
 /**
  * gpk_watch_locked_cb:
  **/
@@ -1364,7 +1413,7 @@ gpk_watch_get_proxy_ftp (GpkWatch *watch)
 
 	/* common case, a direct connection */
 	mode = gconf_client_get_string (watch->priv->gconf_client, "/system/proxy/mode", NULL);
-	if (egg_strequal (mode, "none")) {
+	if (g_strcmp0 (mode, "none") == 0) {
 		egg_debug ("not using session proxy");
 		goto out;
 	}
@@ -1406,7 +1455,7 @@ gpk_watch_get_proxy_http (GpkWatch *watch)
 
 	/* common case, a direct connection */
 	mode = gconf_client_get_string (watch->priv->gconf_client, "/system/proxy/mode", NULL);
-	if (egg_strequal (mode, "none")) {
+	if (g_strcmp0 (mode, "none") == 0) {
 		egg_debug ("not using session proxy");
 		goto out;
 	}
@@ -1580,6 +1629,21 @@ gpk_watch_button_cancel_cb (GtkWidget *widget, GpkWatch *watch)
 	}
 }
 
+/**
+ * gpk_watch_connection_changed_cb:
+ **/
+static void
+gpk_watch_connection_changed_cb (PkConnection *pconnection, gboolean connected, GpkWatch *watch)
+{
+	g_return_if_fail (GPK_IS_WATCH (watch));
+	egg_debug ("connected=%i", connected);
+	if (connected) {
+		gpk_watch_refresh_icon (watch);
+		gpk_watch_refresh_tooltip (watch);
+		gpk_watch_set_proxies (watch);
+	}
+}
+
 /**
  * gpk_watch_init:
  * @watch: This class instance
@@ -1587,14 +1651,12 @@ gpk_watch_button_cancel_cb (GtkWidget *widget, GpkWatch *watch)
 static void
 gpk_watch_init (GpkWatch *watch)
 {
-	PolKitAction *pk_action;
-	PolKitGnomeAction *restart_action;
-
 	watch->priv = GPK_WATCH_GET_PRIVATE (watch);
 	watch->priv->error_details = NULL;
 	watch->priv->notification_cached_messages = NULL;
 	watch->priv->restart = PK_RESTART_ENUM_NONE;
 	watch->priv->hide_warning = FALSE;
+	watch->priv->console = egg_console_kit_new ();
 
 	watch->priv->gconf_client = gconf_client_get_default ();
 
@@ -1650,26 +1712,9 @@ gpk_watch_init (GpkWatch *watch)
 
 	watch->priv->pconnection = pk_connection_new ();
 	g_signal_connect (watch->priv->pconnection, "connection-changed",
-			  G_CALLBACK (pk_connection_changed_cb), watch);
+			  G_CALLBACK (gpk_watch_connection_changed_cb), watch);
 	if (pk_connection_valid (watch->priv->pconnection))
-		pk_connection_changed_cb (watch->priv->pconnection, TRUE, watch);
-
-	pk_action = polkit_action_new ();
-	polkit_action_set_action_id (pk_action, "org.freedesktop.consolekit.system.restart");
-
-	restart_action = polkit_gnome_action_new_default ("restart-system", pk_action,
-							  /* TRANSLATORS: This button restarts the computer after an update */
-							  _("_Restart computer"), NULL);
-	g_object_set (restart_action,
-		      "no-icon-name", "system-shutdown",
-		      "auth-icon-name", "system-shutdown",
-		      "yes-icon-name","system-shutdown",
-		      "self-blocked-icon-name", "system-shutdown",
-		      NULL);
-	polkit_action_unref (pk_action);
-	g_signal_connect (restart_action, "activate",
-			  G_CALLBACK (gpk_watch_restart_cb), NULL);
-	watch->priv->restart_action = restart_action;
+		gpk_watch_connection_changed_cb (watch->priv->pconnection, TRUE, watch);
 
 	/* watch proxy keys */
 	gconf_client_add_dir (watch->priv->gconf_client, GPK_WATCH_GCONF_PROXY_HTTP,
@@ -1719,9 +1764,9 @@ gpk_watch_finalize (GObject *object)
 	g_object_unref (watch->priv->control);
 	g_object_unref (watch->priv->pconnection);
 	g_object_unref (watch->priv->gconf_client);
-	g_object_unref (watch->priv->restart_action);
 	g_object_unref (watch->priv->client_primary);
 	g_object_unref (watch->priv->dialog);
+	g_object_unref (watch->priv->console);
 
 	G_OBJECT_CLASS (gpk_watch_parent_class)->finalize (object);
 }
